@@ -43,6 +43,7 @@ modules = [
     "gelbooru_v01",
     "gelbooru_v02",
     "gfycat",
+    "gofile",
     "hbrowse",
     "hentai2read",
     "hentaicosplays",
@@ -63,6 +64,7 @@ modules = [
     "inkbunny",
     "instagram",
     "issuu",
+    "itaku",
     "kabeuchi",
     "keenspot",
     "kemonoparty",
@@ -105,6 +107,7 @@ modules = [
     "pixiv",
     "pixnet",
     "plurk",
+    "poipiku",
     "pornhub",
     "pururin",
     "reactor",
@@ -125,6 +128,7 @@ modules = [
     "speakerdeck",
     "subscribestar",
     "tapas",
+    "telegraph",
     "toyhouse",
     "tsumino",
     "tumblr",
