@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2018-2021 Mike Fährmann
+# Copyright 2018-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -20,6 +20,7 @@ class ArtstationExtractor(Extractor):
     filename_fmt = "{category}_{id}_{asset[id]}_{title}.{extension}"
     directory_fmt = ("{category}", "{userinfo[username]}")
     archive_fmt = "{asset[id]}"
+    browser = "firefox"
     root = "https://www.artstation.com"
 
     def __init__(self, match):
@@ -29,12 +30,12 @@ class ArtstationExtractor(Extractor):
 
     def items(self):
         data = self.metadata()
-        yield Message.Directory, data
 
         for project in self.projects():
             for asset in self.get_project_assets(project["hash_id"]):
                 asset.update(data)
                 adict = asset["asset"]
+                yield Message.Directory, asset
 
                 if adict["has_embedded_player"] and self.external:
                     player = adict["player_embedded"]
