@@ -82,8 +82,12 @@ class InstagramExtractor(Extractor):
 
         if response.history:
 
-            url = response.request.url
+            url = response.url
             if "/accounts/login/" in url:
+                if self._username:
+                    self.log.debug("Invalidating cached login session for "
+                                   "'%s'", self._username)
+                    _login_impl.invalidate(self._username)
                 page = "login"
             elif "/challenge/" in url:
                 page = "challenge"
@@ -161,55 +165,15 @@ class InstagramExtractor(Extractor):
         return self._pagination_api(endpoint)
 
     def login(self):
+        self._username = None
         if not self._check_cookies(self.cookienames):
             username, password = self._get_auth_info()
             if username:
-                self._update_cookies(self._login_impl(username, password))
+                self._username = username
+                self._update_cookies(_login_impl(self, username, password))
         self.session.cookies.set(
             "csrftoken", self.csrf_token, domain=self.cookiedomain)
 
-    @cache(maxage=360*24*3600, keyarg=1)
-    def _login_impl(self, username, password):
-        self.log.info("Logging in as %s", username)
-
-        url = self.root + "/accounts/login/"
-        page = self.request(url).text
-
-        headers = {
-            "X-Web-Device-Id" : text.extract(page, '"device_id":"', '"')[0],
-            "X-IG-App-ID"     : "936619743392459",
-            "X-ASBD-ID"       : "437806",
-            "X-IG-WWW-Claim"  : "0",
-            "X-Requested-With": "XMLHttpRequest",
-            "Referer"         : url,
-        }
-        url = self.root + "/data/shared_data/"
-        data = self.request(url, headers=headers).json()
-
-        headers["X-CSRFToken"] = data["config"]["csrf_token"]
-        headers["X-Instagram-AJAX"] = data["rollout_hash"]
-        headers["Origin"] = self.root
-        data = {
-            "username"     : username,
-            "enc_password" : "#PWD_INSTAGRAM_BROWSER:0:{}:{}".format(
-                int(time.time()), password),
-            "queryParams"         : "{}",
-            "optIntoOneTap"       : "false",
-            "stopDeletionNonce"   : "",
-            "trustedDeviceRecords": "{}",
-        }
-        url = self.root + "/accounts/login/ajax/"
-        response = self.request(url, method="POST", headers=headers, data=data)
-
-        if not response.json().get("authenticated"):
-            raise exception.AuthenticationError()
-
-        cget = self.session.cookies.get
-        return {
-            name: cget(name)
-            for name in ("sessionid", "mid", "ig_did")
-        }
-
     def _parse_post_graphql(self, post):
         typename = post["__typename"]
 
@@ -286,37 +250,51 @@ class InstagramExtractor(Extractor):
         return data
 
     def _parse_post_api(self, post):
-
-        if "media" in post:
-            media = post["media"]
-            owner = media["user"]
+        if "items" in post:
+            items = post["items"]
+            reel_id = str(post["id"]).rpartition(":")[2]
             data = {
-                "post_id" : media["pk"],
-                "post_shortcode": shortcode_from_id(media["pk"]),
+                "expires": text.parse_timestamp(post.get("expiring_at")),
+                "post_id": reel_id,
+                "post_shortcode": shortcode_from_id(reel_id),
+            }
+        else:
+            data = {
+                "post_id" : post["pk"],
+                "post_shortcode": post["code"],
+                "likes": post["like_count"],
             }
 
-            if "carousel_media" in media:
-                post["items"] = media["carousel_media"]
+            caption = post["caption"]
+            data["description"] = caption["text"] if caption else ""
+
+            tags = self._find_tags(data["description"])
+            if tags:
+                data["tags"] = sorted(set(tags))
+
+            location = post.get("location")
+            if location:
+                slug = location["short_name"].replace(" ", "-").lower()
+                data["location_id"] = location["pk"]
+                data["location_slug"] = slug
+                data["location_url"] = "{}/explore/locations/{}/{}/".format(
+                    self.root, location["pk"], slug)
+
+            if "carousel_media" in post:
+                items = post["carousel_media"]
                 data["sidecar_media_id"] = data["post_id"]
                 data["sidecar_shortcode"] = data["post_shortcode"]
             else:
-                post["items"] = (media,)
-
-        else:
-            reel_id = str(post["id"]).rpartition(":")[2]
-            owner = post["user"]
-            data = {
-                "expires" : text.parse_timestamp(post.get("expiring_at")),
-                "post_id" : reel_id,
-                "post_shortcode": shortcode_from_id(reel_id),
-            }
+                items = (post,)
 
+        owner = post["user"]
         data["owner_id"] = owner["pk"]
         data["username"] = owner.get("username")
         data["fullname"] = owner.get("full_name")
-        data["_files"] = files = []
+        data["post_url"] = "{}/p/{}/".format(self.root, data["post_shortcode"])
 
-        for num, item in enumerate(post["items"], 1):
+        data["_files"] = files = []
+        for num, item in enumerate(items, 1):
 
             image = item["image_versions2"]["candidates"][0]
 
@@ -333,7 +311,8 @@ class InstagramExtractor(Extractor):
             media = {
                 "num"        : num,
                 "date"       : text.parse_timestamp(item.get("taken_at") or
-                                                    media.get("taken_at")),
+                                                    media.get("taken_at") or
+                                                    post.get("taken_at")),
                 "media_id"   : item["pk"],
                 "shortcode"  : (item.get("code") or
                                 shortcode_from_id(item["pk"])),
@@ -342,6 +321,10 @@ class InstagramExtractor(Extractor):
                 "width"      : media["width"],
                 "height"     : media["height"],
             }
+
+            if "expiring_at" in item:
+                media["expires"] = text.parse_timestamp(post["expiring_at"])
+
             self._extract_tagged_users(item, media)
             files.append(media)
 
@@ -385,31 +368,6 @@ class InstagramExtractor(Extractor):
                                          "username" : user["username"],
                                          "full_name": user["full_name"]})
 
-    def _extract_shared_data(self, page):
-        shared_data, pos = text.extract(
-            page, "window._sharedData =", ";</script>")
-        additional_data, pos = text.extract(
-            page, "window.__additionalDataLoaded(", ");</script>", pos)
-
-        data = json.loads(shared_data)
-        if additional_data:
-            next(iter(data["entry_data"].values()))[0] = \
-                json.loads(additional_data.partition(",")[2])
-        return data
-
-    def _get_edge_data(self, user, key):
-        cursor = self.config("cursor")
-        if cursor or not key:
-            return {
-                "edges"    : (),
-                "page_info": {
-                    "end_cursor"   : cursor,
-                    "has_next_page": True,
-                    "_virtual"     : True,
-                },
-            }
-        return user[key]
-
     def _pagination_graphql(self, query_hash, variables):
         cursor = self.config("cursor")
         if cursor:
@@ -436,8 +394,7 @@ class InstagramExtractor(Extractor):
     def _pagination_api(self, endpoint, params=None):
         while True:
             data = self._request_api(endpoint, params=params)
-            for item in data["items"]:
-                yield {"media": item}
+            yield from data["items"]
 
             if not data["more_available"]:
                 return
@@ -446,7 +403,8 @@ class InstagramExtractor(Extractor):
     def _pagination_api_post(self, endpoint, params, post=False):
         while True:
             data = self._request_api(endpoint, method="POST", data=params)
-            yield from data["items"]
+            for item in data["items"]:
+                yield item["media"]
 
             info = data["paging_info"]
             if not info["more_available"]:
@@ -567,21 +525,7 @@ class InstagramTagExtractor(InstagramExtractor):
         return {"tag": text.unquote(self.item)}
 
     def posts(self):
-        url = "{}/explore/tags/{}/".format(self.root, self.item)
-        page = self._extract_shared_data(
-            self.request(url).text)["entry_data"]["TagPage"][0]
-
-        if "data" in page:
-            return self._pagination_sections(page["data"]["recent"])
-
-        hashtag = page["graphql"]["hashtag"]
-        query_hash = "9b498c08113f1e09617a1703c22b2f32"
-        variables = {"tag_name": hashtag["name"], "first": 50}
-        edge = self._get_edge_data(hashtag, "edge_hashtag_to_media")
-        return self._pagination_graphql(query_hash, variables, edge)
-
-    def _pagination_sections(self, info):
-        endpoint = "/v1/tags/instagram/sections/"
+        endpoint = "/v1/tags/{}/sections/".format(self.item)
         data = {
             "include_persistent": "0",
             "max_id" : None,
@@ -591,29 +535,17 @@ class InstagramTagExtractor(InstagramExtractor):
         }
 
         while True:
+            info = self._request_api(endpoint, method="POST", data=data)
+
             for section in info["sections"]:
-                yield from section["layout_content"]["medias"]
+                for media in section["layout_content"]["medias"]:
+                    yield media["media"]
 
             if not info.get("more_available"):
                 return
 
             data["max_id"] = info["next_max_id"]
             data["page"] = info["next_page"]
-            info = self._request_api(endpoint, method="POST", data=data)
-
-    def _pagination_graphql(self, query_hash, variables, data):
-        while True:
-            for edge in data["edges"]:
-                yield edge["node"]
-
-            info = data["page_info"]
-            if not info["has_next_page"]:
-                return
-
-            variables["after"] = self._cursor = info["end_cursor"]
-            self.log.debug("Cursor: %s", self._cursor)
-            data = self._request_graphql(
-                query_hash, variables)["hashtag"]["edge_hashtag_to_media"]
 
 
 class InstagramPostExtractor(InstagramExtractor):
@@ -812,6 +744,49 @@ class InstagramReelsExtractor(InstagramExtractor):
         return self._pagination_api_post(endpoint, data)
 
 
+@cache(maxage=360*24*3600, keyarg=1)
+def _login_impl(extr, username, password):
+    extr.log.info("Logging in as %s", username)
+
+    url = extr.root + "/accounts/login/"
+    page = extr.request(url).text
+
+    headers = {
+        "X-Web-Device-Id" : text.extract(page, '"device_id":"', '"')[0],
+        "X-IG-App-ID"     : "936619743392459",
+        "X-ASBD-ID"       : "437806",
+        "X-IG-WWW-Claim"  : "0",
+        "X-Requested-With": "XMLHttpRequest",
+        "Referer"         : url,
+    }
+    url = extr.root + "/data/shared_data/"
+    data = extr.request(url, headers=headers).json()
+
+    headers["X-CSRFToken"] = data["config"]["csrf_token"]
+    headers["X-Instagram-AJAX"] = data["rollout_hash"]
+    headers["Origin"] = extr.root
+    data = {
+        "username"     : username,
+        "enc_password" : "#PWD_INSTAGRAM_BROWSER:0:{}:{}".format(
+            int(time.time()), password),
+        "queryParams"         : "{}",
+        "optIntoOneTap"       : "false",
+        "stopDeletionNonce"   : "",
+        "trustedDeviceRecords": "{}",
+    }
+    url = extr.root + "/accounts/login/ajax/"
+    response = extr.request(url, method="POST", headers=headers, data=data)
+
+    if not response.json().get("authenticated"):
+        raise exception.AuthenticationError()
+
+    cget = extr.session.cookies.get
+    return {
+        name: cget(name)
+        for name in ("sessionid", "mid", "ig_did")
+    }
+
+
 def id_from_shortcode(shortcode):
     return util.bdecode(shortcode, _ALPHABET)
 
