@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2021 Mike Fährmann
+# Copyright 2021-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -46,7 +46,7 @@ class PhilomenaExtractor(BooruExtractor):
             try:
                 params["filter_id"] = INSTANCES[self.category]["filter_id"]
             except (KeyError, TypeError):
-                pass
+                params["filter_id"] = "2"
 
         while True:
             data = self.request(url, params=params).json()
@@ -58,12 +58,21 @@ class PhilomenaExtractor(BooruExtractor):
 
 
 INSTANCES = {
-    "derpibooru": {"root": "https://derpibooru.org",
-                   "filter_id": "56027"},
-    "ponybooru" : {"root": "https://ponybooru.org",
-                   "filter_id": "2"},
-    "furbooru"  : {"root": "https://furbooru.org",
-                   "filter_id": "2"},
+    "derpibooru": {
+        "root": "https://derpibooru.org",
+        "pattern": r"derpibooru\.org",
+        "filter_id": "56027",
+    },
+    "ponybooru": {
+        "root": "https://ponybooru.org",
+        "pattern": r"ponybooru\.org",
+        "filter_id": "2",
+    },
+    "furbooru": {
+        "root": "https://furbooru.org",
+        "pattern": r"furbooru\.org",
+        "filter_id": "2",
+    },
 }
 
 BASE_PATTERN = PhilomenaExtractor.update(INSTANCES)
@@ -172,7 +181,7 @@ class PhilomenaSearchExtractor(PhilomenaExtractor):
         PhilomenaExtractor.__init__(self, match)
         groups = match.groups()
         if groups[-1]:
-            q = groups[-1]
+            q = groups[-1].replace("+", " ")
             for old, new in (
                 ("-colon-"  , ":"),
                 ("-dash-"   , "-"),
@@ -239,5 +248,5 @@ class PhilomenaGalleryExtractor(PhilomenaExtractor):
     def posts(self):
         gallery_id = "gallery_id:" + self.gallery_id
         url = self.root + "/api/v1/json/search/images"
-        params = {"sd": "desc", "sf": gallery_id, "q" : gallery_id}
+        params = {"sd": "desc", "sf": gallery_id, "q": gallery_id}
         return self._pagination(url, params)
