@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2021 Mike Fährmann
+# Copyright 2021-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -10,10 +10,8 @@
 
 import os
 import re
-import time
 import shutil
 import functools
-from email.utils import mktime_tz, parsedate_tz
 from . import util, formatter, exception
 
 WINDOWS = util.WINDOWS
@@ -76,7 +74,7 @@ class PathFormat():
         self.directory = self.realdirectory = \
             self.filename = self.extension = self.prefix = \
             self.path = self.realpath = self.temppath = ""
-        self.delete = self._create_directory = False
+        self.delete = False
 
         extension_map = config("extension-map")
         if extension_map is None:
@@ -140,7 +138,11 @@ class PathFormat():
 
     def open(self, mode="wb"):
         """Open file and return a corresponding file object"""
-        return open(self.temppath, mode)
+        try:
+            return open(self.temppath, mode)
+        except FileNotFoundError:
+            os.makedirs(self.realdirectory)
+            return open(self.temppath, mode)
 
     def exists(self):
         """Return True if the file exists on disk"""
@@ -177,15 +179,18 @@ class PathFormat():
             self.directory = directory = self.basedirectory
 
         if WINDOWS:
-            # Enable longer-than-260-character paths on Windows
-            directory = "\\\\?\\" + os.path.abspath(directory)
+            # Enable longer-than-260-character paths
+            directory = os.path.abspath(directory)
+            if directory.startswith("\\\\"):
+                directory = "\\\\?\\UNC\\" + directory[2:]
+            else:
+                directory = "\\\\?\\" + directory
 
             # abspath() in Python 3.7+ removes trailing path separators (#402)
             if directory[-1] != sep:
                 directory += sep
 
         self.realdirectory = directory
-        self._create_directory = True
 
     def set_filename(self, kwdict):
         """Set general filename data"""
@@ -277,9 +282,6 @@ class PathFormat():
 
     def build_path(self):
         """Combine directory and filename to full paths"""
-        if self._create_directory:
-            os.makedirs(self.realdirectory, exist_ok=True)
-            self._create_directory = False
         self.filename = filename = self.build_filename(self.kwdict)
         self.path = self.directory + filename
         self.realpath = self.realdirectory + filename
@@ -315,18 +317,19 @@ class PathFormat():
 
         if self.temppath != self.realpath:
             # Move temp file to its actual location
-            try:
-                os.replace(self.temppath, self.realpath)
-            except OSError:
-                shutil.copyfile(self.temppath, self.realpath)
-                os.unlink(self.temppath)
+            while True:
+                try:
+                    os.replace(self.temppath, self.realpath)
+                except FileNotFoundError:
+                    # delayed directory creation
+                    os.makedirs(self.realdirectory)
+                    continue
+                except OSError:
+                    # move across different filesystems
+                    shutil.copyfile(self.temppath, self.realpath)
+                    os.unlink(self.temppath)
+                break
 
         mtime = self.kwdict.get("_mtime")
         if mtime:
-            # Set file modification time
-            try:
-                if isinstance(mtime, str):
-                    mtime = mktime_tz(parsedate_tz(mtime))
-                os.utime(self.realpath, (time.time(), mtime))
-            except Exception:
-                pass
+            util.set_mtime(self.realpath, mtime)
